@@ -1,6 +1,6 @@
 <?php
 
-/*
+/**
  +-----------------------------------------------------------------------+
  | This file is part of the Roundcube PHP suite                          |
  | Copyright (C) 2005-2015, The Roundcube Dev Team                       |
@@ -17,7 +17,6 @@
  +-----------------------------------------------------------------------+
 */
 
-
 /**
  * Roundcube Framework Initialization
  *
@@ -26,7 +25,7 @@
  */
 
 $config = array(
-    'error_reporting'         => E_ALL & ~E_NOTICE & ~E_STRICT & ~E_DEPRECATED,
+    'error_reporting'         => E_ALL & ~E_NOTICE & ~E_STRICT,
     // Some users are not using Installer, so we'll check some
     // critical PHP settings here. Only these, which doesn't provide
     // an error/warning in the logs later. See (#1486307).
@@ -46,15 +45,17 @@ if (php_sapi_name() != 'cli') {
 foreach ($config as $optname => $optval) {
     $ini_optval = filter_var(ini_get($optname), is_bool($optval) ? FILTER_VALIDATE_BOOLEAN : FILTER_VALIDATE_INT);
     if ($optval != $ini_optval && @ini_set($optname, $optval) === false) {
-        $error = "ERROR: Wrong '$optname' option value and it wasn't possible to set it to required value ($optval).\n"
+        $optval = !is_bool($optval) ? $optval : ($optval ? 'On' : 'Off');
+        $error  = "ERROR: Wrong '$optname' option value and it wasn't possible to set it to required value ($optval).\n"
             . "Check your PHP configuration (including php_admin_flag).";
+
         if (defined('STDERR')) fwrite(STDERR, $error); else echo $error;
         exit(1);
     }
 }
 
 // framework constants
-define('RCUBE_VERSION', '1.1.3');
+define('RCUBE_VERSION', '1.2.1');
 define('RCUBE_CHARSET', 'UTF-8');
 
 if (!defined('RCUBE_LIB_DIR')) {
@@ -99,8 +100,7 @@ if (!preg_match($regexp, $path)) {
 spl_autoload_register('rcube_autoload');
 
 // set PEAR error handling (will also load the PEAR main class)
-@PEAR::setErrorHandling(PEAR_ERROR_CALLBACK, 'rcube_pear_error');
-
+PEAR::setErrorHandling(PEAR_ERROR_CALLBACK, 'rcube_pear_error');
 
 
 /**
@@ -133,11 +133,10 @@ function in_array_nocase($needle, $haystack)
     return false;
 }
 
-
 /**
  * Parse a human readable string for a number of bytes.
  *
- * @param string $str  Input string
+ * @param string $str Input string
  *
  * @return float Number of bytes
  */
@@ -168,7 +167,6 @@ function parse_bytes($str)
     return floatval($bytes);
 }
 
-
 /**
  * Make sure the string ends with a slash
  */
@@ -177,7 +175,6 @@ function slashify($str)
   return unslashify($str).'/';
 }
 
-
 /**
  * Remove slashes at the end of the string
  */
@@ -186,11 +183,10 @@ function unslashify($str)
   return preg_replace('/\/+$/', '', $str);
 }
 
-
 /**
  * Returns number of seconds for a specified offset string.
  *
- * @param string $str  String representation of the offset (e.g. 20min, 5h, 2days, 1week)
+ * @param string $str String representation of the offset (e.g. 20min, 5h, 2days, 1week)
  *
  * @return int Number of seconds
  */
@@ -219,21 +215,19 @@ function get_offset_sec($str)
     return $amount;
 }
 
-
 /**
  * Create a unix timestamp with a specified offset from now.
  *
- * @param string $offset_str  String representation of the offset (e.g. 20min, 5h, 2days)
- * @param int    $factor      Factor to multiply with the offset
+ * @param string $offset_str String representation of the offset (e.g. 20min, 5h, 2days)
+ * @param int    $factor     Factor to multiply with the offset
  *
  * @return int Unix timestamp
  */
-function get_offset_time($offset_str, $factor=1)
+function get_offset_time($offset_str, $factor = 1)
 {
     return time() + get_offset_sec($offset_str) * $factor;
 }
 
-
 /**
  * Truncate string if it is longer than the allowed length.
  * Replace the middle or the ending part of a string with a placeholder.
@@ -245,7 +239,7 @@ function get_offset_time($offset_str, $factor=1)
  *
  * @return string Abbreviated string
  */
-function abbreviate_string($str, $maxlength, $placeholder='...', $ending=false)
+function abbreviate_string($str, $maxlength, $placeholder = '...', $ending = false)
 {
     $length = mb_strlen($str);
 
@@ -258,17 +252,18 @@ function abbreviate_string($str, $maxlength, $placeholder='...', $ending=false)
         $first_part_length  = floor(($maxlength - $placeholder_length)/2);
         $second_starting_location = $length - $maxlength + $first_part_length + $placeholder_length;
 
-        $str = mb_substr($str, 0, $first_part_length) . $placeholder . mb_substr($str, $second_starting_location);
+        $prefix = mb_substr($str, 0, $first_part_length);
+        $suffix = mb_substr($str, $second_starting_location);
+        $str    = $prefix . $placeholder . $suffix;
     }
 
     return $str;
 }
 
-
 /**
  * Get all keys from array (recursive).
  *
- * @param array $array  Input array
+ * @param array $array Input array
  *
  * @return array List of array keys
  */
@@ -288,7 +283,6 @@ function array_keys_recursive($array)
     return $keys;
 }
 
-
 /**
  * Remove all non-ascii and non-word chars except ., -, _
  */
@@ -298,7 +292,6 @@ function asciiwords($str, $css_id = false, $replace_with = '')
     return preg_replace("/[^$allowed]/i", $replace_with, $str);
 }
 
-
 /**
  * Check if a string contains only ascii characters
  *
@@ -313,12 +306,11 @@ function is_ascii($str, $control_chars = true)
     return preg_match($regexp, $str) ? false : true;
 }
 
-
 /**
  * Compose a valid representation of name and e-mail address
  *
- * @param string $email  E-mail address
- * @param string $name   Person name
+ * @param string $email E-mail address
+ * @param string $name  Person name
  *
  * @return string Formatted string
  */
@@ -338,7 +330,6 @@ function format_email_recipient($email, $name = '')
     return $email;
 }
 
-
 /**
  * Format e-mail address
  *
@@ -361,7 +352,6 @@ function format_email($email)
     return $email;
 }
 
-
 /**
  * Fix version number so it can be used correctly in version_compare()
  *
@@ -374,44 +364,8 @@ function version_parse($version)
     return str_replace(
         array('-stable', '-git'),
         array('.0', '.99'),
-        $version);
-}
-
-
-/**
- * mbstring replacement functions
- */
-if (!extension_loaded('mbstring'))
-{
-    function mb_strlen($str)
-    {
-        return strlen($str);
-    }
-
-    function mb_strtolower($str)
-    {
-        return strtolower($str);
-    }
-
-    function mb_strtoupper($str)
-    {
-        return strtoupper($str);
-    }
-
-    function mb_substr($str, $start, $len=null)
-    {
-        return substr($str, $start, $len);
-    }
-
-    function mb_strpos($haystack, $needle, $offset=0)
-    {
-        return strpos($haystack, $needle, $offset);
-    }
-
-    function mb_strrpos($haystack, $needle, $offset=0)
-    {
-        return strrpos($haystack, $needle, $offset);
-    }
+        $version
+    );
 }
 
 /**
@@ -425,7 +379,7 @@ if (!function_exists('idn_to_utf8'))
         static $idn, $loaded;
 
         if (!$loaded) {
-            $idn = new Net_IDNA2();
+            $idn    = new Net_IDNA2();
             $loaded = true;
         }
 
@@ -436,6 +390,7 @@ if (!function_exists('idn_to_utf8'))
             catch (Exception $e) {
             }
         }
+
         return $domain;
     }
 }
@@ -447,7 +402,7 @@ if (!function_exists('idn_to_ascii'))
         static $idn, $loaded;
 
         if (!$loaded) {
-            $idn = new Net_IDNA2();
+            $idn    = new Net_IDNA2();
             $loaded = true;
         }
 
@@ -458,6 +413,7 @@ if (!function_exists('idn_to_ascii'))
             catch (Exception $e) {
             }
         }
+
         return $domain;
     }
 }
@@ -470,27 +426,25 @@ if (!function_exists('idn_to_ascii'))
  */
 function rcube_autoload($classname)
 {
-    $filename = preg_replace(
-        array(
-            '/Mail_(.+)/',
-            '/Net_(.+)/',
-            '/Auth_(.+)/',
-            '/^html_.+/',
-            '/^rcube(.*)/'
-        ),
-        array(
-            'Mail/\\1',
-            'Net/\\1',
-            'Auth/\\1',
-            'Roundcube/html',
-            'Roundcube/rcube\\1'
-        ),
-        $classname
-    );
+    if (strpos($classname, 'rcube') === 0) {
+        $classname = 'Roundcube/' . $classname;
+    }
+    else if (strpos($classname, 'html_') === 0 || $classname === 'html') {
+        $classname = 'Roundcube/html';
+    }
+    else if (strpos($classname, 'Mail_') === 0) {
+        $classname = 'Mail/' . substr($classname, 5);
+    }
+    else if (strpos($classname, 'Net_') === 0) {
+        $classname = 'Net/' . substr($classname, 4);
+    }
+    else if (strpos($classname, 'Auth_') === 0) {
+        $classname = 'Auth/' . substr($classname, 5);
+    }
 
-    if ($fp = @fopen("$filename.php", 'r', true)) {
+    if ($fp = @fopen("$classname.php", 'r', true)) {
         fclose($fp);
-        include_once "$filename.php";
+        include_once "$classname.php";
         return true;
     }
 
